@@ -25,11 +25,13 @@
 #include <goffice/graph/gog-axis.h>
 #include <goffice/graph/gog-chart.h>
 #include <goffice/graph/gog-chart-map.h>
+#include <goffice/graph/gog-chart-map-3d.h>
 #include <goffice/graph/gog-data-allocator.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-theme.h>
 #include <goffice/math/go-math.h>
+#include <goffice/utils/go-persist.h>
 
 #include <gsf/gsf-impl-utils.h>
 
@@ -280,8 +282,8 @@ gog_axis_base_get_crossed_axis_type (GogAxisBase *axis_base)
 				crossed_type = GOG_AXIS_RADIAL;
 			break;
 		case GOG_AXIS_SET_X:
-			break;
 		case GOG_AXIS_SET_XYZ:
+			break;
 		case GOG_AXIS_SET_ALL:
 		case GOG_AXIS_SET_NONE:
 		default:
@@ -614,6 +616,7 @@ gog_axis_base_populate_editor (GogObject *gobj,
 
 	if (axis_type == GOG_AXIS_X ||
 	    axis_type == GOG_AXIS_Y ||
+	    axis_type == GOG_AXIS_Z ||
 	    axis_type == GOG_AXIS_RADIAL) {
 		w = glade_xml_get_widget (gui, "padding_spinbutton");
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), axis_base->padding);
@@ -680,71 +683,71 @@ gog_axis_base_class_init (GObjectClass *gobject_klass)
 		g_param_spec_uint ("pos",
 			_("Axis position"),
 			_("Where to position an axis low, high, or crossing"),
-			GOG_AXIS_AT_LOW, GOG_AXIS_CROSS, GOG_AXIS_AT_LOW,
+			GOG_AXIS_AT_LOW, GOG_AXIS_AT_HIGH, GOG_AXIS_AT_LOW,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_POSITION_STR,
 		g_param_spec_string ("pos-str", 
 			_("Axis position (as a string)"),
 			_("Where to position an axis low, high, or crossing"),
 			"low", 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MAJOR_TICK_LABELED,
 		g_param_spec_boolean ("major-tick-labeled", 
 			_("Major labels"),
 			_("Show labels for major ticks"),
 			TRUE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MAJOR_TICK_IN,
 		g_param_spec_boolean ("major-tick-in", 
 			_("Inside major ticks"),
 			_("Major tick marks inside the chart area"),
 			FALSE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MAJOR_TICK_OUT,
 		g_param_spec_boolean ("major-tick-out", 
 			_("Outside major ticks"),
 			_("Major tick marks outside the chart area"),
 			TRUE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MAJOR_TICK_SIZE_PTS,
 		g_param_spec_int ("major-tick-size-pts", 
 			_("Major tick size"),
 			_("Size of the major tick marks, in points"),
 			0, 20, 4, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MINOR_TICK_IN,
 		g_param_spec_boolean ("minor-tick-in", 
 			_("Inside minor ticks"),
 			_("Minor tick marks inside the chart area"),
 			FALSE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MINOR_TICK_OUT,
 		g_param_spec_boolean ("minor-tick-out", 
 			_("Outside minor ticks"),
 			_("Minor tick marks outside the axis"),
 			FALSE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MINOR_TICK_SIZE_PTS,
 		g_param_spec_int ("minor-tick-size-pts", 
 			_("Minor tick size"),
 			_("Size of the minor tick marks, in points"),
 			0, 15, 2, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_CROSS_AXIS_ID,
 		g_param_spec_uint ("cross-axis-id", 
 			_("Cross axis id"),
 			_("Which axis to cross"),
 			0, G_MAXUINT, 0, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_PADDING_PTS,
 		g_param_spec_int ("padding-pts", 
 			_("Axis padding"),
 			_("Distance from axis line to plot area, in points"),
 			-G_MAXINT, G_MAXINT, 0, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 #ifdef GOFFICE_WITH_GTK
 	gog_klass->populate_editor	= gog_axis_base_populate_editor;
@@ -812,7 +815,8 @@ gog_tool_bound_is_valid_axis (GogView *view)
 	GogAxisType type = gog_axis_get_atype (axis_base->axis);
 
 	return (type == GOG_AXIS_X ||
-		type == GOG_AXIS_Y);
+		type == GOG_AXIS_Y ||
+		type == GOG_AXIS_Z);
 
 }
 
@@ -968,7 +972,8 @@ static GogTool gog_axis_tool_stop_bound = {
 typedef enum {
 	GOG_AXIS_BASE_RENDER,
 	GOG_AXIS_BASE_POINT,
-	GOG_AXIS_BASE_PADDING_REQUEST
+	GOG_AXIS_BASE_PADDING_REQUEST,
+	GOG_AXIS_BASE_LABEL_POSITION_REQUEST,
 } GogAxisBaseAction;
 
 static gboolean
@@ -1539,6 +1544,8 @@ x_process (GogAxisBaseAction action, GogView *view, GogViewPadding *padding,
 						x, y, ax, ay, bx - ax, by - ay,
 						GO_SIDE_RIGHT);
 			break;
+		default:
+			break;
 	}
 
 	return FALSE;
@@ -1632,6 +1639,8 @@ xy_process (GogAxisBaseAction action, GogView *view, GogViewPadding *padding,
 						x, y, ax, ay, bx - ax, by - ay,
 						side);
 			break;
+		default:
+			break;
 	}
 
 	return FALSE;
@@ -1735,6 +1744,8 @@ radar_process (GogAxisBaseAction action, GogView *view, GogViewPadding *padding,
 								 side);
 				}
 				break;
+			default:
+				break;
 		}
 		gog_chart_map_free (c_map);
 	} else {
@@ -1759,12 +1770,285 @@ radar_process (GogAxisBaseAction action, GogView *view, GogViewPadding *padding,
 			case GOG_AXIS_BASE_POINT:
 				point = axis_circle_point (x, y, parms->cx, parms->cy, parms->rx, parms->th1);
 				break;
+			default:
+				break;
 		}
 		gog_chart_map_free (c_map);
 	}
 	return point;
 }
 
+static gboolean
+xyz_process (GogAxisBaseAction action, GogView *view, GogViewPadding *padding,
+	    GogViewAllocation const *plot_area, double x, double y)
+{
+	GogAxisBase *axis_base = GOG_AXIS_BASE (view->model);
+	GogAxisBaseView *axis_base_view = GOG_AXIS_BASE_VIEW (view);
+	GogAxisType axis_type = gog_axis_get_atype (axis_base->axis);
+	GogAxis *axis1, *axis2;
+	GogChartMap3D *c_map;
+	GogAxisMap *a_map;
+	GogViewAllocation axis_line_bbox;
+	GSList *axes;
+	GogAxisType perp_axis;
+	GOGeometryOBR obr;
+	GogAxisTick *ticks;
+	double ax, ay, az, bx, by, bz, ox, oy, dist, tmp;
+	double xposition, yposition, zposition;
+	double start, stop;
+	double *px[] = {&ax, &ax, &bx, &bx, &ax, &ax, &bx, &bx};
+	double *py[] = {&ay, &by, &by, &ay, &ay, &by, &by, &ay};
+	double *pz[] = {&az, &az, &az, &az, &bz, &bz, &bz, &bz};
+	double rx[8], ry[8], rz[8];
+	double major_tick_len, minor_tick_len, tick_len;
+	double label_w, label_h;
+
+	/* Note: Anti-clockwise order in each face,
+	 * important for calculating normals */
+	const int faces[] = {
+		3, 2, 1, 0, /* Bottom */
+		4, 5, 6, 7, /* Top */
+		0, 1, 5, 4, /* Left */
+		2, 3, 7, 6, /* Right */
+		1, 2, 6, 5, /* Front */
+		0, 4, 7, 3  /* Back */
+	};
+	int i, tick_nbr, vertex = 0, base = 0;
+	GOGeometrySide side = GO_SIDE_LEFT;
+
+	g_return_val_if_fail (axis_type == GOG_AXIS_X ||
+	                      axis_type == GOG_AXIS_Y ||
+	                      axis_type == GOG_AXIS_Z, FALSE);
+
+	if (!gog_object_is_visible (axis_base->axis))
+		return FALSE;
+
+	if (axis_type == GOG_AXIS_X) {
+		axes  = gog_chart_get_axes (axis_base->chart, GOG_AXIS_Y);
+		axis1 = GOG_AXIS (axes->data);
+		axes  = gog_chart_get_axes (axis_base->chart, GOG_AXIS_Z);
+		axis2 = GOG_AXIS (axes->data);
+		c_map = gog_chart_map_3d_new (axis_base->chart, plot_area,
+			axis_base->axis, axis1, axis2);
+	} else if (axis_type == GOG_AXIS_Y) {
+		axes  = gog_chart_get_axes (axis_base->chart, GOG_AXIS_Z);
+		axis1 = GOG_AXIS (axes->data);
+		axes  = gog_chart_get_axes (axis_base->chart, GOG_AXIS_X);
+		axis2 = GOG_AXIS (axes->data);
+		c_map = gog_chart_map_3d_new (axis_base->chart, plot_area,
+			axis2, axis_base->axis, axis1);
+	} else {
+		axes  = gog_chart_get_axes (axis_base->chart, GOG_AXIS_X);
+		axis1 = GOG_AXIS (axes->data);
+		axes  = gog_chart_get_axes (axis_base->chart, GOG_AXIS_Y);
+		axis2 = GOG_AXIS (axes->data);
+		c_map = gog_chart_map_3d_new (axis_base->chart, plot_area,
+			axis1, axis2, axis_base->axis);
+	}
+
+	a_map = gog_chart_map_3d_get_axis_map (c_map, 0);
+	gog_axis_map_get_bounds (a_map, &ax, &bx);
+	a_map = gog_chart_map_3d_get_axis_map (c_map, 1);
+	gog_axis_map_get_bounds (a_map, &ay, &by);
+	a_map = gog_chart_map_3d_get_axis_map (c_map, 2);
+	gog_axis_map_get_bounds (a_map, &az, &bz);
+
+	/* Projecting vertices */
+	for (i = 0; i < 8; ++i)
+		gog_chart_map_3d_to_view (c_map, *px[i], *py[i], *pz[i],
+		                          &rx[i], &ry[i], &rz[i]);
+
+	/* Determining base plane */
+	dist  = ry[0] + ry[1] + ry[2] + ry[3];
+	for (i = 4; i < 24; i += 4) {
+		tmp = ry[faces[i]] + ry[faces[i + 1]]
+		    + ry[faces[i + 2]] + ry[faces[i + 3]];
+		if (tmp > dist) {
+			dist = tmp;
+			base = i;
+		}
+	}
+	if (base == 0 || base == 4)
+		perp_axis = GOG_AXIS_Z;
+	else if (base == 8 || base == 12)
+		perp_axis = GOG_AXIS_X;
+	else
+		perp_axis = GOG_AXIS_Y;
+
+	/* Position of the centre of the base plane */
+	ox = 0.25 * (rx[faces[base]] + rx[faces[base + 1]]
+	   + rx[faces[base + 2]] + rx[faces[base + 3]]);
+	oy = 0.25 * dist;
+
+	/* Choosing the most distant vertex with respect to the centre
+	 * of the base plane */
+	dist = (rx[faces[base]] - ox);
+	tmp  = (ry[faces[base]] - oy);
+	dist = dist * dist + tmp * tmp;
+	for (i = 1; i < 4; ++i) {
+		double dx = rx[faces[base + i]] - ox;
+		double dy = ry[faces[base + i]] - oy;
+		tmp = dx * dx + dy * dy;
+
+		/* Adding 1e-5 to ensure this inequality remains true
+		 * after changing the ratio */
+		if (tmp > dist + 1e-5) {
+			dist = tmp;
+			vertex = i;
+		}
+	}
+
+	if (axis_type != perp_axis) {
+		/* Here we're choosing one of the nearest neighbours
+		 * of the vertex previously chosen */
+		int pvtx = (vertex + 3) % 4; /* Previous vertex */
+		int nvtx = (vertex + 1) % 4; /* Next vertex */
+		int prev = faces[base + pvtx];
+		int next = faces[base + nvtx];
+		int curr = faces[base + vertex];
+		tmp = (rx[next] - rx[curr]) * (ry[prev] - ry[curr])
+		    - (ry[next] - ry[curr]) * (rx[prev] - rx[curr]);
+		/* If normal is negative, we're choosing vertex which
+		 * is closer to the screen */
+		if ((tmp < 0 && rz[prev] < rz[next])
+		    || (tmp > 0 && rz[prev] > rz[next]))
+			vertex = pvtx;
+		else
+			vertex = nvtx;
+	}
+
+	if (axis_type == GOG_AXIS_Z) {
+		xposition = *px[faces[base + vertex]];
+		yposition = *py[faces[base + vertex]];
+
+		a_map = gog_chart_map_3d_get_axis_map (c_map, 2);
+		gog_axis_map_get_extents (a_map, &start, &stop);
+		gog_chart_map_3d_to_view (c_map, xposition, yposition, start,
+		                          &ax, &ay, NULL);
+		gog_chart_map_3d_to_view (c_map, xposition, yposition, stop,
+		                          &bx, &by, NULL);
+	} else if (axis_type == GOG_AXIS_X) {
+		yposition = *py[faces[base + vertex]];
+		zposition = *pz[faces[base + vertex]];
+
+		a_map = gog_chart_map_3d_get_axis_map (c_map, 0);
+		gog_axis_map_get_extents (a_map, &start, &stop);
+		gog_chart_map_3d_to_view (c_map, start, yposition, zposition,
+		                          &ax, &ay, NULL);
+		gog_chart_map_3d_to_view (c_map, stop, yposition, zposition,
+		                          &bx, &by, NULL);
+	} else {
+		zposition = *pz[faces[base + vertex]];
+		xposition = *px[faces[base + vertex]];
+
+		a_map = gog_chart_map_3d_get_axis_map (c_map, 1);
+		gog_axis_map_get_extents (a_map, &start, &stop);
+		gog_chart_map_3d_to_view (c_map, xposition, start, zposition,
+		                          &ax, &ay, NULL);
+		gog_chart_map_3d_to_view (c_map, xposition, stop, zposition,
+		                          &bx, &by, NULL);
+	}
+
+	if (axis_type == perp_axis) {
+		/* Calculating cross-product of two planar vectors
+		 * to determine "chirality" of the projected axis */
+		tmp = (ax - 0.5 * plot_area->w) * (by - ay)
+		    - (ay - 0.5 * plot_area->h) * (bx - ax);
+	} else {
+		/* Same here, but relative to the centre of the base,
+		 * except for a special case, when its 0 */
+		if ((ax == bx && ax == ox) || (ay == by && ay == oy))
+			tmp = (ax - 0.5 * plot_area->w) * (by - ay)
+			    - (ay - 0.5 * plot_area->h) * (bx - ax);
+		else
+			tmp = (ax - ox) * (by - ay) - (ay - oy) * (bx - ax);
+	}
+	side = (tmp > 0)? GO_SIDE_LEFT : GO_SIDE_RIGHT;
+
+	gog_chart_map_3d_free (c_map);
+
+	switch (action) {
+		case GOG_AXIS_BASE_RENDER:
+			axis_line_render (axis_base, axis_base_view,
+					  view->renderer, 
+					  ax, ay, bx - ax , by - ay, side, -1.,
+					  axis_base->major_tick_labeled, TRUE);
+			break;
+		case GOG_AXIS_BASE_PADDING_REQUEST:
+			axis_line_bbox = axis_line_get_bbox (axis_base,
+				view->renderer, ax, ay, bx - ax, by - ay,
+				side, -1., axis_base->major_tick_labeled);
+			padding->wl = MAX (0., plot_area->x - axis_line_bbox.x);
+			padding->ht = MAX (0., plot_area->y - axis_line_bbox.y);
+			padding->wr = MAX (0., axis_line_bbox.x + axis_line_bbox.w
+			                   - plot_area->x - plot_area->w);
+			padding->hb = MAX (0., axis_line_bbox.y + axis_line_bbox.h
+			                   - plot_area->y - plot_area->h);
+			break;
+		case GOG_AXIS_BASE_POINT:
+			break;
+		case GOG_AXIS_BASE_LABEL_POSITION_REQUEST:
+			/* Calculating unit vector perpendicular to the
+			 * axis projection */
+			if (side == GO_SIDE_RIGHT) {
+				ox = -(by - ay);
+				oy = bx - ax;
+			} else {
+				ox = by - ay;
+				oy = -(bx - ax);
+			}
+			tmp = sqrt (ox * ox + oy * oy);
+			ox *= 1. / tmp;
+			oy *= 1. / tmp;
+
+			/* Axis centre; we'll return it along with offset
+			 * int the GogViewPadding structure */
+			padding->wl = 0.5 * (ax + bx);
+			padding->ht = 0.5 * (ay + by);
+
+			/* Calculating axis label offset */
+			dist = gog_axis_base_get_padding (axis_base);
+			padding->wl += gog_renderer_pt2r_x (view->renderer,
+				dist * ox);
+			padding->ht += gog_renderer_pt2r_y (view->renderer,
+				dist * oy);
+
+			minor_tick_len = gog_renderer_pt2r (view->renderer,
+				axis_base->minor.size_pts);
+			major_tick_len = gog_renderer_pt2r (view->renderer,
+				axis_base->major.size_pts);
+			tick_len = axis_base->major.tick_out ? major_tick_len :
+				(axis_base->minor.tick_out ? minor_tick_len : 0.);
+
+			tick_nbr = gog_axis_get_ticks (axis_base->axis, &ticks);
+
+			gog_renderer_get_text_OBR (view->renderer,
+				"0", TRUE, &obr);
+			tick_len += fabs (obr.w * ox);
+			if (axis_base->major_tick_labeled) {
+				label_w = label_h = 0;
+				for (i = 0; i < tick_nbr; i++) {
+					if (ticks[i].label == NULL)
+						continue;
+					gog_renderer_get_text_OBR (view->renderer,
+						ticks[i].label, TRUE, &obr);
+					if (obr.w > label_w)
+						label_w = obr.w;
+					if (obr.h > label_h)
+						label_h = obr.h;
+				}
+				tick_len += hypot (label_w, label_h);
+			}
+			padding->wr = tick_len * ox;
+			padding->hb = tick_len * oy;
+			break;	
+		default:
+			break;
+	}
+
+	return FALSE;
+}
+
 static gboolean
 gog_axis_base_view_point (GogView *view, double x, double y)
 {
@@ -1800,6 +2084,9 @@ gog_axis_base_view_point (GogView *view, double x, double y)
 		case GOG_AXIS_SET_RADAR:
 			pointed = radar_process (GOG_AXIS_BASE_POINT, view, NULL, plot_area, x, y);
 			break;
+		case GOG_AXIS_SET_XYZ:
+			xyz_process (GOG_AXIS_BASE_POINT, view, NULL, plot_area, x, y);
+			break;
 		default:
 			g_warning ("[AxisBaseView::point] not implemented for this axis set (%i)",
 				   axis_set);
@@ -1834,6 +2121,10 @@ gog_axis_base_view_padding_request (GogView *view, GogViewAllocation const *bbox
 		case GOG_AXIS_SET_RADAR:
 			radar_process (GOG_AXIS_BASE_PADDING_REQUEST, view, padding, bbox, 0., 0.);
 			break;
+		case GOG_AXIS_SET_XYZ:
+			xyz_process (GOG_AXIS_BASE_PADDING_REQUEST, view,
+			             padding, bbox, 0., 0.);
+			break;
 		default:
 			g_warning ("[AxisBaseView::padding_request] not implemented for this axis set (%i)",
 				   axis_set);
@@ -1875,6 +2166,9 @@ gog_axis_base_view_render (GogView *view, GogViewAllocation const *bbox)
 		case GOG_AXIS_SET_RADAR:
 			radar_process (GOG_AXIS_BASE_RENDER, view, NULL, plot_area, 0., 0.);
 			break;
+		case GOG_AXIS_SET_XYZ:
+			xyz_process (GOG_AXIS_BASE_RENDER, view, NULL, plot_area, 0., 0.);
+			break;
 		default:
 			g_warning ("[AxisBaseView::render] not implemented for this axis set (%i)",
 				   axis_set);
@@ -1894,6 +2188,43 @@ gog_axis_base_build_toolkit (GogView *view)
 #endif
 }
 
+void
+gog_axis_base_view_label_position_request (GogView *view,
+                                           GogViewAllocation const *bbox,
+					   GogViewAllocation *pos)
+{
+	GogAxisSet axis_set;
+	GogAxisBase *axis_base = GOG_AXIS_BASE (view->model);
+	GogStyle *style = axis_base->base.style;
+	GogViewPadding padding;
+
+	if (gog_axis_get_atype (axis_base->axis) >= GOG_AXIS_VIRTUAL)
+		return;
+	axis_set = gog_chart_get_axis_set (axis_base->chart);
+	if (axis_set == GOG_AXIS_SET_UNKNOWN)
+		return;
+
+	gog_renderer_push_style (view->renderer, style);
+
+	switch (axis_set & GOG_AXIS_SET_FUNDAMENTAL) {
+		case GOG_AXIS_SET_XYZ:
+			xyz_process (GOG_AXIS_BASE_LABEL_POSITION_REQUEST, view,
+			             &padding, bbox, 0., 0.);
+			break;
+		default:
+			g_warning ("[AxisBaseView::label_position_request] not implemented for this axis set (%i)",
+				   axis_set);
+			break;
+	}
+
+	gog_renderer_pop_style (view->renderer);
+
+	pos->x = padding.wl;
+	pos->y = padding.ht;
+	pos->w = padding.wr;
+	pos->h = padding.hb;
+}
+
 static void
 gog_axis_base_view_class_init (GogAxisBaseViewClass *gview_klass)
 {
