@@ -30,6 +30,8 @@
 #include <goffice/utils/go-color.h>
 #include <goffice/math/go-math.h>
 #include <goffice/utils/go-marker.h>
+#include <goffice/utils/go-path.h>
+#include <goffice/utils/go-persist.h>
 #include <goffice/utils/go-units.h>
 
 #include <gsf/gsf-impl-utils.h>
@@ -206,13 +208,13 @@ gog_legend_class_init (GogLegendClass *klass)
 			_("Swatch Size pts"),
 			_("size of the swatches in pts."),
 			0, G_MAXDOUBLE, 0, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, LEGEND_SWATCH_PADDING_PTS,
 		g_param_spec_double ("swatch-padding-pts", 
 			_("Swatch Padding pts"),
 			_("padding between the swatches in pts."),
 			0, G_MAXDOUBLE, 0, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 }
 
 static void
@@ -373,33 +375,40 @@ static void
 cb_swatch_scale (unsigned i, GogStyle const *style, char const *name,
 		 SwatchScaleClosure *data)
 {
+	GogStyleLine const *line = NULL;
 	double size;
 	double maximum, scale;
-       
+
 	size = go_marker_get_size (style->marker.mark);
 	if (data->size_max < size)
 		data->size_max = size;
 	if (data->size_min > size)
 		data->size_min = size;
 
-	if (style->interesting_fields & GOG_STYLE_LINE &&
-	    style->line.width > data->hairline_width) {
-		if (style->line.dash_type != GO_LINE_NONE && style->line.dash_type != GO_LINE_SOLID)
-			maximum = data->line_length / (GLV_SAMPLE_DASH_LENGTH 
-						       * go_line_dash_get_length (style->line.dash_type));
-		else
-			maximum = style->line.width;
+	if (style->interesting_fields & GOG_STYLE_LINE)
+		line = &style->line;
+	else if (style->interesting_fields & GOG_STYLE_OUTLINE)
+		line = &style->outline;
 
-		if (maximum > data->line_width_max)
-			maximum = data->line_width_max;
+	if (line == NULL ||
+	    line->width <= data->hairline_width)
+		return;
 
-		if (maximum > data->hairline_width) 
-			scale = (maximum - data->hairline_width) / (style->line.width - data->hairline_width);
-		else 
-			scale = 0;
-		if (data->line_scale > scale)
-			data->line_scale = scale;
-	}
+	if (line->dash_type != GO_LINE_NONE && line->dash_type != GO_LINE_SOLID)
+		maximum = data->line_length / (GLV_SAMPLE_DASH_LENGTH
+					       * go_line_dash_get_length (line->dash_type));
+	else
+		maximum = line->width;
+
+	if (maximum > data->line_width_max)
+		maximum = data->line_width_max;
+
+	if (maximum > data->hairline_width)
+		scale = (maximum - data->hairline_width) / (line->width - data->hairline_width);
+	else
+		scale = 0;
+	if (data->line_scale > scale)
+		data->line_scale = scale;
 }
 
 typedef struct {
@@ -409,7 +418,6 @@ typedef struct {
 	double element_step_x, element_step_y;
 	double block_step_x, block_step_y;
 	GogViewAllocation swatch;
-	ArtVpath line_path[3];
 	double swatch_scale_a, swatch_scale_b;
 	double line_scale_a, line_scale_b;
 	double hairline_width;
@@ -424,7 +432,8 @@ cb_render_elements (unsigned index, GogStyle const *base_style, char const *name
 	GogRenderer *renderer = view->renderer;
 	GogStyle *style = NULL;
 	GogViewAllocation pos, rectangle;
-	double half_width;
+	double half_width, y;
+	GOPath *line_path;
 
 	if (data->count > 0) {
 		if ((data->count % glv->element_per_blocks) != 0) {
@@ -436,10 +445,10 @@ cb_render_elements (unsigned index, GogStyle const *base_style, char const *name
 		}
 	}
 	data->count++;
-	
+
 	if (base_style->interesting_fields & GOG_STYLE_LINE) { /* line and marker */
 		style = gog_style_dup (base_style);
-		go_marker_set_size (style->marker.mark, 
+		go_marker_set_size (style->marker.mark,
 				    go_marker_get_size (style->marker.mark) *
 				    data->swatch_scale_a + data->swatch_scale_b);
 		if (style->line.width > data->hairline_width)
@@ -447,17 +456,26 @@ cb_render_elements (unsigned index, GogStyle const *base_style, char const *name
 
 		gog_renderer_push_style (renderer, style);
 		half_width = 0.5 * gog_renderer_line_size (renderer, style->line.width);
-		data->line_path[0].x = data->x + half_width;
-		data->line_path[1].x = data->x + data->swatch.w * GLV_LINE_LENGTH_EM - half_width;
-		data->line_path[0].y = 
-		data->line_path[1].y = data->y + glv->element_height / 2.;
-		gog_renderer_draw_path (renderer, data->line_path);
-		gog_renderer_draw_marker (renderer, data->x + data->swatch.w  * GLV_LINE_LENGTH_EM * 0.5, 
-					  data->line_path[0].y);
+		line_path = go_path_new ();
+		y = data->y + glv->element_height / 2.;
+		go_path_move_to (line_path, data->x + half_width, y);
+		go_path_line_to (line_path, data->x + data->swatch.w * GLV_LINE_LENGTH_EM - half_width, y);
+		if (style->interesting_fields & GOG_STYLE_FILL) {
+			rectangle.x = data->x - half_width;
+			rectangle.y = y;
+			rectangle.w = data->swatch.w * GLV_LINE_LENGTH_EM + 2.0 * half_width;
+			rectangle.h = glv->element_height / 2.0;
+			gog_renderer_fill_rectangle (renderer, &rectangle);
+		}
+		gog_renderer_stroke_serie (renderer, line_path);
+		go_path_free (line_path);
+		gog_renderer_draw_marker (renderer, data->x + data->swatch.w  * GLV_LINE_LENGTH_EM * 0.5, y);
 	} else {					/* area swatch */
 		style = gog_style_dup (base_style);
-		style->outline.width = 0; 
-		style->outline.color = RGBA_BLACK;
+		if (style->outline.width > data->hairline_width)
+			style->outline.width =
+				0.5 * (data->hairline_width + style->outline.width) * data->line_scale_a +
+				data->line_scale_b;
 
 		rectangle = data->swatch;
 		rectangle.x += data->x;
@@ -530,11 +548,11 @@ gog_legend_view_render (GogView *v, GogViewAllocation const *bbox)
 
 	data.hairline_width = hairline_width;
 
-	if (glv->uses_lines) {
+/*	if (glv->uses_lines) {
 		data.line_path[0].code = ART_MOVETO;
 		data.line_path[1].code = ART_LINETO;
 		data.line_path[2].code = ART_END;
-	}
+	}*/
 	
 	data.count = 0;
 	data.view = v;
