@@ -27,6 +27,7 @@
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-error-bar.h>
 #include <goffice/data/go-data.h>
+#include <goffice/utils/go-persist.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
@@ -229,7 +230,7 @@ gog_series_element_class_init (GogSeriesElementClass *klass)
 			_("Index"),
 			_("Index of the corresponding data element"),
 			0, G_MAXINT, 0,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT | GOG_PARAM_FORCE_SAVE));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT | GOG_PARAM_FORCE_SAVE));
 }
 
 GSF_CLASS (GogSeriesElement, gog_series_element,
@@ -617,19 +618,19 @@ gog_series_class_init (GogSeriesClass *klass)
 			_("Has-legend"),
 			_("Should the series show up in legends"),
 			TRUE,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
         g_object_class_install_property (gobject_klass, SERIES_PROP_INTERPOLATION,
 		 g_param_spec_string ("interpolation",
 			_("Interpolation"),
 			_("Type of line interpolation"),
 			"linear",
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, SERIES_PROP_FILL_TYPE,
 		g_param_spec_string ("fill-type", 
 			_("Fill type"),
 			_("How to fill the area"),
 			"invalid",
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	klass->valid_fill_type_list = NULL;
 
@@ -682,6 +683,7 @@ gog_series_dataset_set_dim (GogDataset *set, int dim_i,
 	GogGraph *graph = gog_object_get_graph (GOG_OBJECT (series));
 
 	g_return_if_fail (IS_GOG_PLOT (series->plot));
+	g_return_if_fail (dim_i >= -1);
 
 	if (dim_i < 0) {
 		char *name = NULL;
@@ -697,6 +699,8 @@ gog_series_dataset_set_dim (GogDataset *set, int dim_i,
 	/* clone shared dimensions into other series in the plot, and
 	 * invalidate if necessary */
 	desc = &series->plot->desc.series;
+	g_return_if_fail (dim_i < (int) desc->num_dim);
+
 	if (desc->dim[dim_i].is_shared) {
 		GSList *ptr = series->plot->series;
 
@@ -825,10 +829,10 @@ gog_series_has_legend (GogSeries const *series)
 /**
  * gog_series_set_index :
  * @series: #GogSeries
- * @ind:
- * @is_manual:
+ * @ind: >= 0 assigns a new index, < 0 resets to auto
+ * @is_manual: gboolean
  *
- * If @index >= 0 attempt to assign the new index.  Auto
+ * If @ind >= 0 attempt to assign the new index.  Auto
  * indicies (@is_manual == FALSE) will not override the current
  * index if it is manual.  An @index < 0, will reset the index to
  * automatic and potentially queue a revaluation of the parent
@@ -890,7 +894,7 @@ gog_series_set_name (GogSeries *series, GODataScalar *name_src, GError **err)
 /**
  * gog_series_set_dim :
  * @series : #GogSeries
- * @dim_i :
+ * @dim_i : Which dimension
  * @val   : #GOData
  * @err : optional #Gerror pointer
  *
