@@ -21,6 +21,7 @@
  */
 
 #include <goffice/goffice-config.h>
+#include <goffice/app/go-doc.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-styled-object.h>
 #include <goffice/math/go-math.h>
@@ -30,6 +31,7 @@
 #include <goffice/utils/go-image.h>
 #include <goffice/utils/go-line.h>
 #include <goffice/utils/go-marker.h>
+#include <goffice/utils/go-persist.h>
 
 #ifdef GOFFICE_WITH_GTK
 #include <goffice/gtk/goffice-gtk.h>
@@ -1399,7 +1401,8 @@ gog_style_fill_sax_save (GsfXMLOut *output, GogStyle const *style)
 		/* FIXME : type is not a good characterization */
 		gsf_xml_out_add_cstr_unchecked (output, "type",
 			image_tiling_as_str (style->fill.image.type));
-/* TODO save the pixels */
+		gsf_xml_out_add_cstr (output, "name", go_image_get_name (style->fill.image.image));
+		go_doc_save_image ((GODoc *) g_object_get_data (G_OBJECT (gsf_xml_out_get_output (output)), "document"), go_image_get_name (style->fill.image.image));
 		gsf_xml_out_end_element (output);
 		break;
 	default:
@@ -1605,7 +1608,7 @@ gog_style_font_sax_save (GsfXMLOut *output, GogStyle const *style)
 	gsf_xml_out_start_element (output, "font");
 	go_xml_out_add_color (output, "color", style->font.color);
 	str = go_font_as_str (style->font.font);
-	gsf_xml_out_add_cstr_unchecked (output, "font", str);
+	gsf_xml_out_add_cstr (output, "font", str);
 	g_free (str);
 	gsf_xml_out_add_bool (output, "auto-scale", style->font.auto_scale);
 	gsf_xml_out_end_element (output);
@@ -1627,12 +1630,13 @@ static void
 gog_style_text_layout_sax_save (GsfXMLOut *output, GogStyle const *style)
 {
 	gsf_xml_out_start_element (output, "text_layout");
-	gsf_xml_out_add_float (output, "angle", style->text_layout.angle, 6);
+	if (!style->text_layout.auto_angle)
+		gsf_xml_out_add_float (output, "angle", style->text_layout.angle, 6);
 	gsf_xml_out_end_element (output);
 }
 
 static gboolean
-gog_style_persist_dom_load (GogPersist *gp, xmlNode *node)
+gog_style_persist_dom_load (GOPersist *gp, xmlNode *node)
 {
 	GogStyle *style = GOG_STYLE (gp);
 	xmlNode *ptr;
@@ -1716,13 +1720,15 @@ static void
 gog_style_sax_load_fill_image (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	GogStyle *style = GOG_STYLE (xin->user_state);
+	GODoc *doc = (GODoc *) g_object_get_data (G_OBJECT (gsf_xml_in_get_input (xin)), "document");
 	g_return_if_fail (style->fill.type == GOG_FILL_STYLE_IMAGE);
+	g_return_if_fail (IS_GO_DOC (doc));
 	/* TODO: load the pixels */
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
 		if (0 == strcmp (attrs[0], "type")) {
 			style->fill.image.type = str_as_image_tiling (attrs[1]);
-			break;
-		}
+		} else if (0 == strcmp (attrs[0], "name"))
+			style->fill.image.image = g_object_ref (go_doc_image_fetch (doc, attrs[1]));
 }
 
 static void
@@ -1792,7 +1798,7 @@ gog_style_sax_load_text_layout (GsfXMLIn *xin, xmlChar const **attrs)
 }
 
 static void
-gog_style_persist_sax_save (GogPersist const *gp, GsfXMLOut *output)
+gog_style_persist_sax_save (GOPersist const *gp, GsfXMLOut *output)
 {
 	GogStyle const *style = GOG_STYLE (gp);
 
@@ -1814,7 +1820,7 @@ gog_style_persist_sax_save (GogPersist const *gp, GsfXMLOut *output)
 }
 
 static void
-gog_style_persist_prep_sax (GogPersist *gp, GsfXMLIn *xin, xmlChar const **attrs)
+gog_style_persist_prep_sax (GOPersist *gp, GsfXMLIn *xin, xmlChar const **attrs)
 {
 	static GsfXMLInNode const dtd[] = {
 		GSF_XML_IN_NODE 	(STYLE, STYLE, 
@@ -1842,7 +1848,7 @@ gog_style_persist_prep_sax (GogPersist *gp, GsfXMLIn *xin, xmlChar const **attrs
 					 &gog_style_sax_load_fill_gradient, NULL),
 		GSF_XML_IN_NODE 	(STYLE_FILL, FILL_IMAGE, 
 					 -1, "image", 
-					 GSF_XML_CONTENT, 
+					 GSF_XML_NO_CONTENT, 
 					 &gog_style_sax_load_fill_image, NULL),
 		GSF_XML_IN_NODE 	(STYLE, STYLE_MARKER,	
 					 -1, "marker", 
@@ -1866,7 +1872,7 @@ gog_style_persist_prep_sax (GogPersist *gp, GsfXMLIn *xin, xmlChar const **attrs
 }
 
 static void
-gog_style_persist_init (GogPersistClass *iface)
+gog_style_persist_init (GOPersistClass *iface)
 {
 	iface->dom_load = gog_style_persist_dom_load;
 	iface->prep_sax = gog_style_persist_prep_sax;
@@ -1876,7 +1882,7 @@ gog_style_persist_init (GogPersistClass *iface)
 GSF_CLASS_FULL (GogStyle, gog_style,
 		NULL, NULL, gog_style_class_init, NULL,
 		gog_style_init, G_TYPE_OBJECT, 0,
-		GSF_INTERFACE (gog_style_persist_init, GOG_PERSIST_TYPE))
+		GSF_INTERFACE (gog_style_persist_init, GO_PERSIST_TYPE))
 
 gboolean
 gog_style_is_different_size (GogStyle const *a, GogStyle const *b)
@@ -2115,6 +2121,11 @@ gog_style_create_cairo_pattern (GogStyle const *style, cairo_t *cr)
 				return cairo_pattern_create_rgba (1, 1, 1, 1);
 
 			cr_pattern = go_image_create_cairo_pattern (style->fill.image.image);
+			if (cr_pattern == NULL) {
+				/* don't reference anymore an invalid image */
+				((GogStyle *) style)->fill.image.image = NULL;
+				return cairo_pattern_create_rgba (1, 1, 1, 1);
+			}
 			g_object_get (style->fill.image.image, "width", &w, "height", &h, NULL);
 			cairo_pattern_set_extend (cr_pattern, CAIRO_EXTEND_REPEAT);
 			switch (style->fill.image.type) {
