@@ -72,7 +72,7 @@ enum {
 #define GO_COMBO_STACK(o)	G_TYPE_CHECK_INSTANCE_CAST (o, GO_COMBO_STACK_TYPE, GOComboStack)
 #define IS_GO_COMBO_STACK(o)	G_TYPE_CHECK_INSTANCE_TYPE (o, GO_COMBO_STACK_TYPE)
 
-static GtkType go_combo_stack_get_type   (void);
+static GType go_combo_stack_get_type   (void);
 static guint go_combo_stack_signals [LAST_SIGNAL] = { 0, };
 
 static void
@@ -243,6 +243,8 @@ typedef GtkToolItemClass GOToolComboStackClass;
 #define IS_GO_TOOL_COMBO_STACK(o)	(G_TYPE_CHECK_INSTANCE_TYPE (o, GO_TOOL_COMBO_STACK_TYPE))
 
 static GType go_tool_combo_stack_get_type (void);
+
+#ifndef HAVE_GTK_TOOL_ITEM_SET_TOOLTIP_TEXT
 static gboolean
 go_tool_combo_stack_set_tooltip (GtkToolItem *tool_item, GtkTooltips *tooltips,
 				 char const *tip_text,
@@ -250,14 +252,17 @@ go_tool_combo_stack_set_tooltip (GtkToolItem *tool_item, GtkTooltips *tooltips,
 {
 	GOToolComboStack *self = (GOToolComboStack *)tool_item;
 	go_combo_box_set_tooltip (GO_COMBO_BOX (self->combo), tooltips,
-		tip_text, tip_private);
+				  tip_text, tip_private);
 	return TRUE;
 }
+#endif
 
 static void
 go_tool_combo_stack_class_init (GtkToolItemClass *tool_item_klass)
 {
+#ifndef HAVE_GTK_TOOL_ITEM_SET_TOOLTIP_TEXT
 	tool_item_klass->set_tooltip = go_tool_combo_stack_set_tooltip;
+#endif
 }
 
 static GSF_CLASS (GOToolComboStack, go_tool_combo_stack,
@@ -424,7 +429,7 @@ go_action_combo_stack_push (GOActionComboStack *a,
 /**
  * go_action_combo_stack_pop :
  * @act : #GOActionComboStack
- * @n :
+ * @n : count
  *
  * Shorten list @act by removing @n off the top (or fewer if the list is
  * shorter)
@@ -448,7 +453,7 @@ go_action_combo_stack_pop (GOActionComboStack *a, unsigned n)
 /**
  * go_action_combo_stack_truncate :
  * @act : #GOActionComboStack
- * @n : 
+ * @n : maximum length
  *
  * Ensure that list @act is no longer than @n, dropping any extra off the
  * bottom.
